@@ -1,5 +1,5 @@
 import traceback
-from liquer.state import State, EvaluationException
+from liquer.state import State, EvaluationException, vars_clone
 from liquer.parser import (
     encode,
     decode,
@@ -18,6 +18,8 @@ from liquer.state_types import encode_state_data, state_types_registry
 import os.path
 from datetime import datetime
 import json
+from liquer.constants import Status
+import liquer.util as util
 
 
 def find_queries_in_template(template: str, prefix: str, sufix: str):
@@ -33,13 +35,41 @@ def find_queries_in_template(template: str, prefix: str, sufix: str):
         yield template, None
 
 
+class Vars(dict):
+    def __init__(self, *arg, **kwarg):
+        super().__init__(*arg, **kwarg)
+        self._modified_vars = set()
+
+    def __getattr__(self, name):
+        return self[name]
+
+    def __setattr__(self, name, value):
+        if name == "_modified_vars":
+            super().__setattr__(name, value)
+        else:
+            self._modified_vars.add(name)
+            self[name] = value
+
+    def __getstate__(self):
+        return (self._modified_vars, dict(self))
+
+    def __setstate__(self, x):
+        mv, d = x
+        super().__setattr__("_modified_vars", mv)
+        self.clear()
+        self.update(d)
+        
+    def get_modified(self):
+        return {key: self[key] for key in self._modified_vars}
+
+
 class Context(object):
     def __init__(self, parent_context=None, debug=False):
         self.parent_context = parent_context  # parent context - when in child context
 
         self.raw_query = None  # String with the evaluated query
         self.query = None  # Query object of the evaluated query
-        self.status = "none"  # Status: ready, error...
+        self.status = Status.NONE  # Status: ready, error...
         self.is_error = False  # True is evaluation failed
         self.started = ""  # Evaluation start time
         self.created = ""  # Created time (evaluation finished)
@@ -61,18 +91,23 @@ class Context(object):
         self.caching = True  # caching of the results enabled
         self.enable_store_metadata = True  # flag to controll storing of metadata
 
-        self.last_report_time=None # internal time stamp of the last report
-        self._progress_indicator_identifier=1 # counter for creating unique progress identifiers
+        self.last_report_time = None  # internal time stamp of the last report
+        self._progress_indicator_identifier = (
+            1  # counter for creating unique progress identifiers
+        )
+
+        self.vars = Vars(vars_clone())
+        self.html_preview = ""
 
     def can_report(self):
         if self.last_report_time is None:
             self.last_report_time = datetime.now()
         return True
-        return (datetime.now()-self.last_report_time).total_seconds()>0.1
+        return (datetime.now() - self.last_report_time).total_seconds() > 0.1
 
     def metadata(self):
         return dict(
-            status=self.status,
+            status=self.status.value,
             query=self.raw_query,
             parent_query=self.parent_query,
             argument_queries=self.argument_queries,
@@ -87,8 +122,15 @@ class Context(object):
             updated=self.now(),
             created=self.created,
             caching=self.caching,
+            vars=dict(self.vars),
+            html_preview=self.html_preview,
         )
 
+    def set_html_preview(self, html):
+        self.html_preview = html
+        self.store_metadata()
+        return self
+
     def enable_cache(self, enable=True):
         self.caching = enable
         return self
@@ -107,9 +149,14 @@ class Context(object):
                 self.last_report_time = datetime.now()
 
     def new_progress_indicator(self):
-        self._progress_indicator_identifier+=1
+        self._progress_indicator_identifier += 1
         self.progress_indicators.append(
-            dict(id=self._progress_indicator_identifier, step=0, total_steps=None, message="")
+            dict(
+                id=self._progress_indicator_identifier,
+                step=0,
+                total_steps=None,
+                message="",
+            )
         )
         return self._progress_indicator_identifier
 
@@ -119,7 +166,6 @@ class Context(object):
         ]
         if self.parent_context is not None:
             self.parent_context.remove_child_progress(self.raw_query)
-        
 
     def progress_indicator_index(self, identifier):
         if identifier is None:
@@ -134,18 +180,20 @@ class Context(object):
         return None
 
     def now(self):
-        return datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+        return util.now()
 
-    def progress(self, step=0, total_steps=None, message="", identifier=None, autoremove=True):
+    def progress(
+        self, step=0, total_steps=None, message="", identifier=None, autoremove=True
+    ):
         index = self.progress_indicator_index(identifier)
-            
+
         progress = dict(step=step, total_steps=total_steps, message=message)
         self.progress_indicators[index].update(progress)
 
-        removed=False
-        if autoremove and total_steps is not None and step>=total_steps:
+        removed = False
+        if autoremove and total_steps is not None and step >= total_steps:
             self.remove_progress_indicator(index)
-            removed=True
+            removed = True
 
         self.store_metadata()
 
@@ -156,23 +204,31 @@ class Context(object):
                 d = dict(origin=self.raw_query, **progress)
                 self.parent_context.log_child_progress(d)
 
-    def progress_iter(self, iterator):
+    def progress_iter(self, iterator, show_value=False):
         try:
             total_steps = len(iterator)
         except:
             total_steps = None
         identifier = self.new_progress_indicator()
-        for i,x in enumerate(iterator):
-            self.progress(i, total_steps=total_steps, message=f"{i+1}" if total_steps is None else f"{i+1}/{total_steps}", identifier=identifier)
+        for i, x in enumerate(iterator):
+            if total_steps is None:
+                message = f"{x} ({i+1})" if show_value else f"{i+1}"
+            else:
+                message = (
+                    f"{x} ({i+1}/{total_steps})"
+                    if show_value
+                    else f"{i+1}/{total_steps}"
+                )
+            self.progress(
+                i, total_steps=total_steps, message=message, identifier=identifier
+            )
             yield x
         self.remove_progress_indicator(identifier)
-        
+
     def remove_child_progress(self, origin):
         "Remove all child progress indicators from a given origin"
         self.child_progress_indicators = [
-            x
-            for x in self.child_progress_indicators
-            if x.get("origin") != origin
+            x for x in self.child_progress_indicators if x.get("origin") != origin
         ]
         self.store_metadata()
         if self.parent_context is not None:
@@ -196,7 +252,7 @@ class Context(object):
         self.log.append(d)
         if "message" in d:
             self.message = d["message"]
-        self.store_metadata(force = (d.get("kind")=="error"))
+        self.store_metadata(force=(d.get("kind") == "error"))
         if self.parent_context is not None:
             if d.get("origin") is None:
                 d = dict(origin=self.raw_query, **d)
@@ -227,7 +283,7 @@ class Context(object):
     def error(self, message, position=None, query=None):
         """Log an error message"""
         self.is_error = True
-        self.status = "error"
+        self.status = Status.ERROR
         if position is None:
             print("ERROR:    ", message)
         else:
@@ -249,7 +305,7 @@ class Context(object):
     def exception(self, message, traceback, position=None, query=None):
         """Log an exception"""
         self.is_error = True
-        self.status = "error"
+        self.status = Status.ERROR
         if position is None:
             print("EXCEPTION:", message)
         else:
@@ -303,7 +359,7 @@ class Context(object):
 
     def evaluate_action(self, state: State, action, cache=None):
         self.debug(f"EVALUATE ACTION '{action}' on '{state.query}'")
-        self.status = "action"
+        self.status = Status.EVALUATION
         self.store_metadata(force=True)
         cache = cache or self.cache()
         cr = self.command_registry()
@@ -327,7 +383,7 @@ class Context(object):
             self.error(f"Unknown action: {action.name} at {action.position}")
         else:
             parameters = []
-            self.status = "evaluate arguments"
+            self.status = Status.EVALUATING_DEPENDENCIES
             self.store_metadata(force=True)
             for p in action.parameters:
                 if isinstance(p, StringActionParameter):
@@ -369,18 +425,18 @@ class Context(object):
                         pp = ExpandedActionParameter(value.get(), p.link, p.position)
                         parameters.append(pp)
                 else:
-                    self.status = "error"
+                    self.status = Status.ERROR
                     self.store_metadata(force=True)
                     raise EvaluationException(
                         f"Unknown parameter type {type(p)} in {action.name}",
                         position=action.position,
                         query=self.raw_query,
                     )
-            self.status = "evaluate action"
+            self.status = Status.EVALUATION
             self.store_metadata(force=True)
 
             try:
-                
+
                 state = command(old_state, *parameters, context=self)
                 assert type(state.metadata) is dict
             except EvaluationException as ee:
@@ -445,18 +501,27 @@ class Context(object):
                 metadata.get("attributes", {}), **cmd_metadata.attributes
             )
 
-        metadata["caching"]=metadata.get("caching",True) and state.metadata.get("caching",True)
-        is_error=state.is_error
-        self.status = "action done"
+        metadata["caching"] = metadata.get("caching", True) and state.metadata.get(
+            "caching", True
+        )
+        is_error = state.is_error
 
         if is_error:
+            self.status = Status.ERROR
             self.info(f"Action {action.encode()} at {action.position} failed")
             state.metadata.update(metadata)
-            state.status="error"
-            state.is_error=True
+            state.status = Status.ERROR.value
+            state.is_error = True
         else:
+            self.status = Status.FINISHED
             self.info(f"Action {action.encode()} at {action.position} completed")
+            state_vars = dict(self.vars)
+            state_vars.update(state.vars)
+            state_vars.update(self.vars.get_modified())
+            self.vars = Vars(state_vars)
+            metadata["vars"] = dict(state_vars)
             state.metadata.update(metadata)
+
         state.set_volatile(is_volatile or state.is_volatile())
 
         cache.store_metadata(state.metadata)
@@ -500,8 +565,9 @@ class Context(object):
     def evaluate(self, query, cache=None, description=None):
         """Evaluate query, returns a State, cache the output in supplied cache"""
         self.enable_store_metadata = False  # Prevents overwriting cache with metadata
-        self.status = "started"
+        self.status = Status.EVALUATION
         self.debug(f"EVALUATE {query}")
+        self.vars = Vars(vars_clone())
         if self.query is not None:
             self.enable_store_metadata = True
             print(f"Subquery {query} called from {self.query.encode()}")
@@ -511,9 +577,9 @@ class Context(object):
             self.log_subquery(query=query, description=description)
             if state.is_error:
                 print("Subquery failed")
-                for d in state.metadata.get("log",[]):
+                for d in state.metadata.get("log", []):
                     self.log_dict(d)
-#            self.enable_store_metadata = True
+            #            self.enable_store_metadata = True
             self.store_metadata(force=True)
             self.enable_store_metadata = False
             return state
@@ -545,19 +611,18 @@ class Context(object):
             self.debug(f"INITIAL STATE")
         else:
             self.parent_query = p.encode()
-            self.status = "evaluate parent"
+            self.status = Status.EVALUATING_PARENT
             self.store_metadata(force=True)
             state = self.child_context().evaluate(p, cache=cache)
-
         if state.is_error:
-            self.status = "error"
+            self.status = Status.ERROR
             self.store_metadata()
             state = state.next_state()
             state.query = query.encode()
             state.metadata["created"] = self.now()
             self.debug(f"ERROR in '{state.query}'")
             return state
-
+        self.vars = Vars(state.vars)
         if r is None:
             self.debug(f"RETURN '{query}' AFTER EMPTY ACTION ON '{state.query}'")
             state.query = query.encode()
@@ -572,7 +637,6 @@ class Context(object):
         state.query = query.encode()
         state.metadata["created"] = self.now()
         state.metadata["status"] = "ready"
-        
 
         if (
             state.metadata.get("caching", True)
@@ -589,7 +653,7 @@ class Context(object):
             else:
                 print("REMOVE CACHE", state.query)
                 if not cache.remove(state.query):
-                    self.status = "obsolete"
+                    self.status = Status.OBSOLETE
                     self.store_metadata()
 
         return state
@@ -610,13 +674,17 @@ class Context(object):
 
         path = target_file
         if path is None:
-            if  state.metadata.get("extension") is None:
+            if state.metadata.get("extension") is None:
                 b, mime, typeid = encode_state_data(data)
                 path = t.default_filename()
             else:
-                b, mime, typeid = encode_state_data(data, extension=state.metadata["extension"])
+                b, mime, typeid = encode_state_data(
+                    data, extension=state.metadata["extension"]
+                )
                 path = (
-                    t.default_filename() if state.metadata.get("filename") is None else state.metadata["filename"]
+                    t.default_filename()
+                    if state.metadata.get("filename") is None
+                    else state.metadata["filename"]
                 )
             if target_directory is not None:
                 path = os.path.join(target_directory, path)
