@@ -489,6 +489,15 @@ class Query(object):
                 return None, None
         else:
             return None, None
+    def short(self):
+        _, r = self.predecessor()
+        if r is None:
+            q=str(self)
+            if len(self)>30:
+                q="..."+q[-30:]
+            return q
+        else:
+            return str(r)
 
     def all_predecessors(self):
         qp, qr = self, None
@@ -590,7 +599,7 @@ def _expand_entity_parse_action(s, loc, toks):
     position = Position.from_loc(loc, s)
     return LinkActionParameter(toks[0], position=position)
 
-expand_entity = (Literal("~X~").suppress() + parse_query + end_entity.suppress()).setParseAction(_expand_entity_parse_action).setName("space_entity")
+expand_entity = (Literal("~X~").suppress() + parse_query + end_entity.suppress()).setParseAction(_expand_entity_parse_action).setName("expand_entity")
 
 entities = (
     tilde_entity
